/* ----------------------------- MNI Header -----------------------------------
@NAME       : minc_error.c
@DESCRIPTION: File containing routines to do error handling for MINC package.
              Should be called through macros in minc_private.h
@GLOBALS    : 
@CALLS      : 
@CREATED    : August 7, 1992 (Peter Neelin)
@MODIFIED   : 
 * $Log: minc_error.c,v $
 * Revision 6.2  2001/04/17 18:40:13  neelin
 * Modifications to work with NetCDF 3.x
 * In particular, changed NC_LONG to NC_INT (and corresponding longs to ints).
 * Changed NC_UNSPECIFIED to NC_NAT.
 * A few fixes to the configure script.
 *
 * Revision 6.1  1999/10/19 14:45:09  neelin
 * Fixed Log subsitutions for CVS
 *
 * Revision 6.0  1997/09/12 13:24:54  neelin
 * Release of minc version 0.6
 *
 * Revision 5.0  1997/08/21  13:25:53  neelin
 * Release of minc version 0.5
 *
 * Revision 4.0  1997/05/07  20:07:52  neelin
 * Release of minc version 0.4
 *
 * Revision 3.0  1995/05/15  19:33:12  neelin
 * Release of minc version 0.3
 *
 * Revision 2.0  1994/09/28  10:38:04  neelin
 * Release of minc version 0.2
 *
 * Revision 1.7  94/09/28  10:37:16  neelin
 * Pre-release
 * 
 * Revision 1.6  93/08/11  12:06:24  neelin
 * Added RCS logging in source.
 * 
@COPYRIGHT  :
              Copyright 1993 Peter Neelin, McConnell Brain Imaging Centre, 
              Montreal Neurological Institute, McGill University.
              Permission to use, copy, modify, and distribute this
              software and its documentation for any purpose and without
              fee is hereby granted, provided that the above copyright
              notice appear in all copies.  The author and McGill University
              make no representations about the suitability of this
              software for any purpose.  It is provided "as is" without
              express or implied warranty.
---------------------------------------------------------------------------- */

#ifndef lint
static char rcsid[] = "$Header: /software/source/minc/cvsroot/minc/libsrc/minc_error.c,v 6.2 2001/04/17 18:40:13 neelin Exp $ MINC (MNI)";
#endif

#include <errno.h>
#include <stdarg.h>
#include <minc_private.h>

semiprivate int MI_save_routine_name(char *name)
{
   MI_ROUTINE_VAR = name; 
   return(TRUE);
}
semiprivate int MI_return(void)
{ 
   return(TRUE); 
}
semiprivate int MI_return_error(void)
{ 
   MI_LOG_PKG_ERROR2(0, "MINC package entry point"); 
   return(TRUE);
}
semiprivate void MI_log_pkg_error2(int p1, char *p2)
{
  (void) fprintf(stderr, "%s: ", MI_ROUTINE_VAR);
  (void) fprintf(stderr, p2);
  (void) fputc('\n', stderr);
  (void) fflush(stderr);
}
semiprivate void MI_log_pkg_error3(int p1, char *p2, char *p3)
{ 
  (void) fprintf(stderr, "%s: ", MI_ROUTINE_VAR);
  (void) fprintf(stderr, p2, p3);
  (void) fputc('\n', stderr);
  (void) fflush(stderr);
}
semiprivate void MI_log_sys_error1(char *p1)
{
   char *message;
   int errnum = errno;

   (void) fprintf(stderr, "%s", MI_ROUTINE_VAR);
   (void) fprintf(stderr, p1);
   if (errnum == 0) {
     (void) fputc('\n', stderr);
   }
   else {
     message = strerror(errnum);
     if (message == NULL) message = "Unknown error";
     (void) fprintf(stderr, ": %s\n", message);
   }
   (void) fflush(stderr);
}

/* By default, print all messages of severity warning, or worse.
 */
static struct {
    int level;
    char prog[128];
    FILE *fp;
} _MI_log = {
    MI_MSG_WARNING, {""}, NULL
};

public void milog_init(const char *name)
{
    char *fname_str = getenv("MINC_LOGFILE");
    char *level_str = getenv("MINC_LOGLEVEL");

    if (fname_str == NULL) {
	_MI_log.fp = stderr;
    }
    else if (!strcmp(fname_str, "stdout") || !strcmp(fname_str, "-")) {
	_MI_log.fp = stdout;
    }
    else {
	if (*fname_str == '+') {
	    _MI_log.fp = fopen(fname_str + 1, "w+");
	}
	else {
	    _MI_log.fp = fopen(fname_str, "w");
	}
    }

    if (level_str == NULL) {
	_MI_log.level = MI_MSG_WARNING;
    }
    else {
	_MI_log.level = atoi(level_str);
    }

    strncpy(_MI_log.prog, name, sizeof (_MI_log.prog));
}

public int milog_set_verbosity(int lvl)
{
    int lvl_prev = _MI_log.level;
    _MI_log.level = lvl;
    return (lvl_prev);
}

public int milog_message(int lvl, char *fmt, ...)
{
    va_list ap;

    if (_MI_log.fp == NULL) {
	_MI_log.fp = stderr;
    }

    /* Log the message if the call depth is 1 and the message priority
     * is less than the configured priority.  Always log fatal errors.
     */
    if ((MI_CALL_DEPTH == 1 && lvl <= _MI_log.level) || lvl == MI_MSG_FATAL) {
	if (_MI_log.prog[0] != '\0') {
	    fprintf(_MI_log.fp, "%s ", _MI_log.prog);
	}
	fprintf(_MI_log.fp, "(from %s): ", MI_ROUTINE_VAR);
	va_start(ap, fmt);
	vfprintf(_MI_log.fp, fmt, ap);
	va_end(ap);
	fprintf(_MI_log.fp, "\n");
	fflush(_MI_log.fp);
    }

    /* For fatal messages, give up and exit.
     */
    if (lvl == MI_MSG_FATAL) {
	exit(-1);
    }

    return (MI_ERROR);		/* Just for convenience */
}

